# TMVA BDT discriminators
# Training with hyporopt

import ROOT
 
ROOT.TMVA.Tools.Instance()

# hyperopt define a search space
# HyperOpt must be installed     sudo pip install hyperopt
# see webpage        https://hyperopt.github.io/hyperopt/
from hyperopt import fmin, tpe, hp, STATUS_OK


 
# note that it seems to be mandatory to have an
# output file, just passing None to TMVA::Factory(..)
# does not work. Make sure you don't overwrite an
# existing file.

fin = ROOT.TFile.Open("ntuple.root")
ntuple = fin.Get('ntuple')
ntuple.Print()

fout = ROOT.TFile("TMVA.root","RECREATE")

dataloader = ROOT.TMVA.DataLoader("dataset");


dataloader.AddVariable("x","F")
dataloader.AddVariable("y","F") 
 
dataloader.AddSignalTree(ntuple,1.0)
dataloader.AddBackgroundTree(ntuple,1.0)
 
# cuts defining the signal and background sample
sigCut = ROOT.TCut("signal > 0.5")
bgCut = ROOT.TCut("signal <= 0.5")
 
# I reduce the number of inut events to 2000 signal and 2000 background for training and for test 
dataloader.PrepareTrainingAndTestTree(sigCut,   # signal events
                                   bgCut,    # background events
                                   ":".join([
                                        "nTrain_Signal=2000",
                                        "nTrain_Background=2000",
                                        "nTest_Signal=2000",
                                        "nTest_Background=2000",
                                        "SplitMode=Random",
                                        "NormMode=NumEvents",
                                        "!V"
                                       ]))



iopt=0

#define objective, i.e. what we want to minimize. In this case it is the 1-ROC integral 
def objective(params):
    #non-linear !!!  BDT
    print "NTrees = ",params['NTrees'],"  MaxDepth = ",params['MaxDepth']
    global iopt

    # create factory
    factory = ROOT.TMVA.Factory("TMVAClassification", fout,
                            ":".join([
                            "!V",
                            "!Silent",
                            "Color",
                            "DrawProgressBar",
                            "Transformations=I;D;P;G,D",
                            "AnalysisType=Classification"]
                            ))

    # book BDT with a number to plot all final ROC curves
    factory.BookMethod(dataloader, ROOT.TMVA.Types.kBDT, "BDT"+str(iopt),"!H:!V:NTrees="+str(int(params['NTrees']))+":MaxDepth="+str(int(params['MaxDepth']))+":BoostType=AdaBoost:AdaBoostBeta=0.5:SeparationType=GiniIndex:nCuts=20:PruneMethod=NoPruning")
       
    # Train, test, evaluate all methods   
    factory.TrainAllMethods()
    factory.TestAllMethods() 
    factory.EvaluateAllMethods()
    ROCint = 1.-factory.GetMethod(dataloader.GetName(),"BDT"+str(iopt)).GetROCIntegral()
    print "ROCIntegral = ",ROCint
    factory.GetMethod(dataloader.GetName(),"BDT"+str(iopt)).Reset()

#    factory.DeleteAllMethods()    
#    factory.Delete()
    del factory
    iopt=iopt+1             
    # return 1-ROCintegral
    print "return",ROCint
    return {'loss': ROCint, 'status': STATUS_OK }
  
  
# space of parameters:   NTrees:   10 - 2000
#                        MaxDepth of a tree 1-20
fspace = { 'NTrees': hp.uniform("NTrees", 10, 2000),
           'MaxDepth': hp.uniform("MaxDepth", 1, 20),}   

# the main call to minimization function 
best = fmin(objective,space=fspace,algo=tpe.suggest, max_evals=40)

print best
# print results
print int(best['NTrees'])," ",int(best['MaxDepth'])


factory = ROOT.TMVA.Factory("TMVAClassification", fout,
                            ":".join([
                            "!V",
                            "!Silent",
                            "Color",
                            "DrawProgressBar",
                            "Transformations=I;D;P;G,D",
                            "AnalysisType=Classification"]
                            ))

#train BDT with the best hyperparameters
factory.BookMethod(dataloader, ROOT.TMVA.Types.kBDT, "BDT","!H:!V:NTrees="+str(int(best['NTrees']))+":MaxDepth=10:BoostType=AdaBoost:AdaBoostBeta=0.5:SeparationType=GiniIndex:nCuts=20:PruneMethod=NoPruning")

factory.TrainAllMethods()
factory.TestAllMethods()
factory.EvaluateAllMethods()

factory.Delete()

#TMVA::TMVAGui()
#ROOT.TMVA.TMVAGui()




