# TMVA linear discriminators

import ROOT
 
ROOT.TMVA.Tools.Instance()
 
# note that it seems to be mandatory to have an
# output file, just passing None to TMVA::Factory(..)
# does not work. Make sure you don't overwrite an
# existing file.

fin = ROOT.TFile.Open("ntuple.root")
ntuple = fin.Get('ntuple')
ntuple.Print()

fout = ROOT.TFile("TMVA.root","RECREATE")

dataloader = ROOT.TMVA.DataLoader("dataset");
factory = ROOT.TMVA.Factory("TMVAClassification", fout,
                            ":".join([
                                "!V",
                                "!Silent",
                                "Color",
                                "DrawProgressBar",
                                "Transformations=I;D;P;G,D",
                                "AnalysisType=Classification"]
                                     ))

dataloader.AddVariable("x","F")
dataloader.AddVariable("y","F") 
 
dataloader.AddSignalTree(ntuple,1.0)
dataloader.AddBackgroundTree(ntuple,1.0)
 
# cuts defining the signal and background sample
sigCut = ROOT.TCut("signal > 0.5")
bgCut = ROOT.TCut("signal <= 0.5")
 
dataloader.PrepareTrainingAndTestTree(sigCut,   # signal events
                                   bgCut,    # background events
                                   ":".join([
                                        "nTrain_Signal=2000",
                                        "nTrain_Background=2000",
                                        "nTest_Signal=2000",
                                        "nTest_Background=2000",
                                        "SplitMode=Random",
                                        "NormMode=NumEvents",
                                        "!V"
                                       ]))





# Setup cross-validation 
###cv = ROOT.TMVA.CrossValidation(dataloader)

cv = ROOT.TMVA.CrossValidation("TMVACrossValidation",dataloader,fout,
                                        ":".join([
                                        "!Silent",
                                        "ModelPersistence",
                                        "AnalysisType=Classification",
                                        "NumFolds=5",
                                        "!V"
                                       ]))

#non-linear !!!  BDT
cv.BookMethod( ROOT.TMVA.Types.kBDT, "BDT",
                   ":".join([
                       "!H",
                       "!V",
                       "NTrees=100",
                       #"nEventsMin=150",
                       "MaxDepth=4",
                       "BoostType=AdaBoost",
                       "AdaBoostBeta=0.5",
                       "SeparationType=GiniIndex",
                       "nCuts=20",
                       "PruneMethod=NoPruning",
                       ]))



#Run cross-validation and print results
cv.SetNumFolds(10)
cv.Evaluate()
CVResult = cv.GetResults()
print CVResult
CVResult[0].Print()
CVResult[0].Draw()
CVResult[0].GetTrainEff01Values()
CVResult[0].GetEff01Values()
CVResult[0].GetTrainEff10Values()
CVResult[0].GetEff10Values()
CVResult[0].GetTrainEff30Values()
CVResult[0].GetEff30Values()
print 'Eff. area values:'
print CVResult[0].GetEffAreaValues()
print 'ROC average'
print CVResult[0].GetROCAverage()
#print 'Sep values:'
#CVResult.GetSepValues()
## AllGraphs is a TMultiGraph
AllGraphs = CVResult[0].GetROCCurves()
#AllGraphs.Print()
## Get histogram used to draw axis, and change max and min
hist = AllGraphs.GetHistogram()
#hist.GetXaxis().SetRangeUser(0.8,1.0)
#hist.GetYaxis().SetRangeUser(0.8,1.0)
#AllGraphs.Draw()


try:
    input("Press enter to continue")
except SyntaxError:
    pass



